/* subtract data from A to B*/
/* the usage is 'mkprj dark direct transmit projection.tif' */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include "tiffio.h"

#define INTEL

#define HiPic_Header_Size 64

struct HiPic_Header{
	char			head[2];
	short			comment_length;
	short			width;
	short			height;
	short			x_offset;
	short			y_offset;
	short			type;
	char			reserved[63 - 13];
	char			*comment;
};
typedef struct HiPic_Header Header;

static unsigned short	*data;
static float		*data32;
static long	Nx, Ny;
static int			BPS;
static char			*desc;

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

/*----------------------------------------------------------------------*/

void Store32TiffFile(char *wname, int wX, int wY, int wBPS, float *data32, char *wdesc)
{
	TIFF *image;
	long i;

	image = TIFFOpen(wname, "w");

	TIFFSetField(image, TIFFTAG_IMAGEWIDTH, wX);
	TIFFSetField(image, TIFFTAG_IMAGELENGTH, wY);
	TIFFSetField(image, TIFFTAG_BITSPERSAMPLE, 32);
	TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
	TIFFSetField(image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
	TIFFSetField(image, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_IEEEFP );
	TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL, 1);
	TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, 1);
	TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(image, TIFFTAG_IMAGEDESCRIPTION, wdesc);
	TIFFSetField(image, TIFFTAG_ARTIST, "p_rec");
//	TIFFSetField(image, TIFFTAG_MINSAMPLEVALUE, mmmin );
//	TIFFSetField(image, TIFFTAG_MAXSAMPLEVALUE, mmmax );

	for (i = 0; i<wY; i++) {
		TIFFWriteRawStrip(image, i, data32 + i*wX, wX * sizeof(float));
	}

	TIFFClose(image);
}


/*----------------------------------------------------------------------*/

int read_hipic(fname, h)
char	*fname;
Header	*h;
{
		short	x_size, y_size;
		int		j;
		FILE	*fi;

//open input files
		if((fi = fopen(fname,"rb")) == NULL)
		{
			printf("can not open %s for input\n", fname);
			free(h->comment); 
			return(-10);
		}
// read comment and image data from file1
		if (fread(h, sizeof(char), HiPic_Header_Size, fi) != HiPic_Header_Size)
		{
			fprintf(stderr, "EOF in %s (header)\n", fname);
			free(h->comment); 
			return(-1);
		}
		if (strncmp(h->head, "IM", 2))
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "This is not HiPIc image file.\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-11);
		}

		h->comment = (char *) malloc(h->comment_length + 1);
		if (h->comment == NULL)
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "Connot allocate memory (HiPic_Header.comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-12);
		}
		if (fread(h->comment, sizeof(char), h->comment_length, fi) != h->comment_length)
		{ 
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "EOF found (comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-13);
		}

		h->comment[h->comment_length] = '\0';

		x_size = h->width;
		y_size = h->height;

		data = (short *) malloc(x_size * y_size * sizeof(short));
		if ((j = fread(data, sizeof(short), x_size*y_size, fi)) != x_size*y_size)
		{
			fclose(fi);
			free(h->comment); 
			fprintf(stderr, " Error reading %s at %d\n", fname, j);
			fprintf(stderr, "File name = %s\n", fname); 
			return(1);
		}

// check for correct reading of HiPic file
//printf("-------------------------------\n");
//printf("%s\n", fname);
//printf("%s\n", h->head);
//printf("%d\n", h->comment_length);
//printf("%d\n", h->width);
//printf("%d\n", h->height);
//printf("%d\n", h->x_offset);
//printf("%d\n", h->y_offset);
//printf("%d\n", h->type);
//printf("%s\n", h->reserved);
//printf("%s\n", h->comment);
//printf("-------------------------------\n");

		fclose(fi);
		return(0);
	}

/*----------------------------------------------------------------------*/

int	main(argc,argv)
int		argc;
char	*argv[];
{
	Header	h;
	unsigned short	*image1, *image2, *outimg, *dark, I1, I2, d, I;
	int		x_size, y_size, num;
	long	i, j, ii, jj, jx;
	double	t1, t2, tx, t21, I0;
	float	pmin, pmax;
	char	p1[15], *comm;
	double	base;
	double	div;
	long	val;

	if(argc < 5){   // when bad number of parameters
		printf("The usage is '%s dark direct transmit projection.tif'\n", argv[0]);
		exit(-1);
	}
	

	for(i=1;i<4;++i){
		if ((j = read_hipic(argv[i], &h)) != 0){
			printf("something wrong -- return value is %d", i);
			exit(-1);
		}
		x_size = h.width;
		y_size = h.height;
		if(i==1){
			dark = (unsigned short *) malloc(x_size*y_size*sizeof(unsigned short));
			for(jj=0;jj<x_size*y_size;++jj) *(dark+jj)=*(data+jj);
		}
		if(i==2){
			image1 = (unsigned short *) malloc(x_size*y_size*sizeof(unsigned short));
			for(jj=0;jj<x_size*y_size;++jj) *(image1+jj)=*(data+jj);
		}
		if(i==3){
			image2 = (unsigned short *) malloc(x_size*y_size*sizeof(unsigned short));
			for(jj=0;jj<x_size*y_size;++jj) *(image2+jj)=*(data+jj);
		}
	}

//output file name
	sprintf(p1,argv[4]);

	data32=(float *) malloc(x_size*y_size*sizeof(float));
	pmin=100000.0;
	pmax=-100000.0;
// calc. max and min.
	for(jj=0;jj<x_size*y_size;++jj){
		d=*(dark+jj);
		I0=*(image1+jj)-d;
		I=*(image2+jj)-d;
		*(data32+jj)=(float)log(I0/I);
		if(pmin>*(data32+jj)) pmin=*(data32+jj);
		if(pmax<*(data32+jj)) pmax=*(data32+jj);
//		printf("%d\t%d\t%d\t%lf\t%d\t%d\t%lf\n", jj, d, I, I0, I1, I2, *(p+jj));
	}

    if ((comm=(char *)malloc(100))==NULL)
		Error("comment memory allocation error.");
	sprintf(comm,"%lf\t%lf",pmax,pmin);
	(void)Store32TiffFile(p1, x_size, y_size, 32, data32, comm);
	printf("Stored to:\t%s\t%s\n",p1,comm);

//fin:
	free(h.comment);
	free(comm);
	free(image1);
	free(image2);
	free(dark);
	free(data32);
	
	return (0);
}

