/* subtract data from A to B*/
/* the usage is 'mkprj dark direct transmit projection.tif' */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include "tiffio.h"
#include "cell.h"
#include "rif.h"

#define INTEL

static unsigned short	*data;
static float		*data32;
static long	Nx, Ny;
static int			BPS;
static char			*desc;

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

/*----------------------------------------------------------------------*/

void Store32TiffFile(char *wname, int wX, int wY, int wBPS, float *data32, char *wdesc)
{
	TIFF *image;
	long i;

	image = TIFFOpen(wname, "w");

	TIFFSetField(image, TIFFTAG_IMAGEWIDTH, wX);
	TIFFSetField(image, TIFFTAG_IMAGELENGTH, wY);
	TIFFSetField(image, TIFFTAG_BITSPERSAMPLE, 32);
	TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
	TIFFSetField(image, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
	TIFFSetField(image, TIFFTAG_SAMPLEFORMAT, SAMPLEFORMAT_IEEEFP );
	TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL, 1);
	TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, 1);
	TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
	TIFFSetField(image, TIFFTAG_IMAGEDESCRIPTION, wdesc);
	TIFFSetField(image, TIFFTAG_ARTIST, "p_rec");
//	TIFFSetField(image, TIFFTAG_MINSAMPLEVALUE, mmmin );
//	TIFFSetField(image, TIFFTAG_MAXSAMPLEVALUE, mmmax );

	for (i = 0; i<wY; i++) {
		TIFFWriteRawStrip(image, i, data32 + i*wX, wX * sizeof(float));
	}

	TIFFClose(image);
}


/*----------------------------------------------------------------------*/

int	main(argc,argv)
int		argc;
char	*argv[];
{
	unsigned short	*image1, *image2, *outimg, *dark, I1, I2, d, I;
	int		jx, jy;
	int		x_size, y_size, num;
	long	i, j, ii, jj;
	double	t1, t2, tx, t21, I0;
	float	pmin, pmax;
	char	p1[15], *comm;
	double	base;
	double	div;
	long	val;
	Cell	**cell;

	if(argc < 5){   // when bad number of parameters
		printf("The usage is '%s dark direct transmit projection.tif'\n", argv[0]);
		exit(-1);
	}
	

	for(i=1;i<4;++i){
		ReadImageFile(argv[i],&x_size,&y_size,&BPS,&cell,&desc);
		if(i==1){
			dark = (unsigned short *) malloc(x_size*y_size*sizeof(unsigned short));
			for(jy=0;jy<y_size;++jy){
				for(jx=0;jx<x_size;++jx){
					*(dark+x_size*jy+jx)=cell[jy][jx];
				}
			}
			for (jy=0; jy<Ny ; jy++) free(cell[jy]);
			free(cell);
		}
		if(i==2){
			image1 = (unsigned short *) malloc(x_size*y_size*sizeof(unsigned short));
			for(jy=0;jy<y_size;++jy){
				for(jx=0;jx<x_size;++jx){
					*(image1+x_size*jy+jx)=cell[jy][jx];
				}
			}
			for (jy=0; jy<Ny ; jy++) free(cell[jy]);
			free(cell);
		}
		if(i==3){
			image2 = (unsigned short *) malloc(x_size*y_size*sizeof(unsigned short));
			for(jy=0;jy<y_size;++jy){
				for(jx=0;jx<x_size;++jx){
					*(image2+x_size*jy+jx)=cell[jy][jx];
				}
			}
			for (jy=0; jy<Ny ; jy++) free(cell[jy]);
			free(cell);
		}
	}

//output file name
	sprintf(p1,argv[4]);

	data32=(float *) malloc(x_size*y_size*sizeof(float));
	pmin=100000.0;
	pmax=-100000.0;
// calc. max and min.
	for(jj=0;jj<x_size*y_size;++jj){
		d=*(dark+jj);
		I0=*(image1+jj)-d;
		I=*(image2+jj)-d;
		*(data32+jj)=(float)log(I0/I);
		if(pmin>*(data32+jj)) pmin=*(data32+jj);
		if(pmax<*(data32+jj)) pmax=*(data32+jj);
//		printf("%d\t%d\t%d\t%lf\t%d\t%d\t%lf\n", jj, d, I, I0, I1, I2, *(p+jj));
	}

    if ((comm=(char *)malloc(100))==NULL)
		Error("comment memory allocation error.");
	sprintf(comm,"%lf\t%lf",pmax,pmin);
	(void)Store32TiffFile(p1, x_size, y_size, 32, data32, comm);
	printf("Stored to:\t%s\t%s\n",p1,comm);

//fin:
	free(comm);
	free(image1);
	free(image2);
	free(dark);
	free(data32);
	
	return (0);
}

